                                    ## Demo R code.R ##

## Demography: research note ##
## Method for redistributing misclassified causes of death ##
## Online Supplementary Appendix S2 ##

## Example of the application of the Ledermann's method ##
## Subset of two years: 1979 and 2016 ##
rm(list=objects())
library(plyr)
library(readxl)

## Set your working directory and put 'Demo Dataset.xlsx' and 'Population.xlsx' there ##
setwd("~/")

## Loading death counts ##
dta<-as.data.frame(read_xlsx("Demo Dataset.xlsx"))

dta1<-dta ## keeping initial data for the later comparison

## Dropping national-total (999) for the moment ##
dta<-dta[dta$Region!=999,]

## I. Implementing the Ledermann's method of redistribution ## 

## Step 1. Calculating the share of each cause in the total (all causes combined)  ## 

## Function calculating CoD shares within each stratum ##
## Causes - CoD items to consider. All (1:10) by default ##  
ShareCD<-function (X,Causes=1:10){
  tmp<-subset(X,Cause %in% Causes,select=c(Cause,Deaths)) 
  TOT<-sum(tmp[,2])+0.000001; tmp[,2]<-tmp[,2]/TOT
  names(tmp)<-c("Cause","Share")
  return(tmp) }

## Appplying 'ShareCD' to the entire dataset##
dtaP<-ddply(dta,.(Year,Sex,Region,Age), function(i) ShareCD (X=i[,],Causes=1:10))

#head(dtaP); tail(dtaP); dim(dtaP)
#write.csv(dtaP,"Shares French dept.csv",row.names=F)  

## Step 2. Getting dissimulation coefficients ##

## Function estimating dissimulation coefficients using the Ledermann's method ##
## X - data frame on which the operation should be performed
## FROM - what to redistribute 
## TO - where to redistribute ##
## pvlim - minimum p-value (default is 1 - disregard significance) ##
## smooth - should coefficients be smoothed across ages ## 
## smpar - smoothing parameter (degree of smoothing to be applied) 
CoeffLed<-function (X,FROM=10,TO=1:9,pvlim=1,smooth=T,smpar=0.6){
  
  ILD<-subset(X,Cause %in% FROM,select=c(Region,Cause,Age,Share)) ## ill-defined ##
  ILD<-aggregate(ILD$Share,by=list(Region=ILD$Region,TAge=ILD$Age),FUN=sum) ## sum of ill-defined
  names(ILD)[3]<-"ShareILD"
  d<-subset(X,Cause %in% TO,select=c(Region,Cause,Age,Share)) ## well-defined 
  
  
  Age_names<-unique(d$Age)
  Coeff_m<-matrix(NA,length(TO),length(Age_names))
  Coeff_end<-NULL
  for(a in 1:length(Age_names))
  {
    ILD_a<-ILD$ShareILD[ILD$TAge==Age_names[a]]
    Coeff_a<-NULL
    for (i in TO)
    {
      CD<-d$Share[d$Cause==i&d$Age==Age_names[a]]
      ILD_a<-ILD$ShareILD[ILD$TAge==Age_names[a]]
      model<-lm(CD~ILD_a) ## linear model
      Reg<-summary(model)[[4]]
      pvalue<-round(Reg[2,4],3) ## extracting slopes and p-values from the model, positive and NaN set to zero 
      slope<-round(Reg[2,1],5)
      tmp<-as.data.frame(cbind(slope,pvalue))
      tmp$Cause<-i
      if(is.na(pvalue)==TRUE)
      {
        Coeff_m[i,a]<-0 
        tmp[,1]<-0
      }else if(pvalue<pvlim&&slope<0)
      {
        Coeff_m[i,a]<-slope
        tmp[,1]<-slope
        tmp[,2]<-pvalue
      }else{
        Coeff_m[i,a]<-0
        tmp[,1]<-0
      }
      tmp<-cbind(Age_names[a],tmp)
      Coeff_a<-rbind(Coeff_a,tmp)
    }
    Coeff_a$slope[is.na(Coeff_a$slope)]<-0
    Coeff_a$slope[is.nan(Coeff_a$slope)]<-0
    TOT<-sum(Coeff_a$slope)+0.000001
    Coeff_a$Coeff_raw<-round(Coeff_a$slope/TOT,3) # computation of raw coefficients
    Coeff_end<-rbind(Coeff_end,Coeff_a)
  }
  #Loop to introduce two constraints: (1) Sum of coeff = 1 ; (2) Smoothing over ages
  for(z in 1:4)
  {
    TOT<-apply(Coeff_m,2,sum)
    Coeff_m<-sweep(Coeff_m,2,TOT,"/")
    Coeff_m[is.nan(Coeff_m)]<-0
    
    if (smooth){# smoothing procedure
    
      for(i in TO){
        
      Temp<-loess(Coeff_m[i,]~Age_names,span=smpar) 
      Coeff_m[i,]<-round(predict(Temp,Age_names),3)
      }
      Coeff_m[Coeff_m<0]<-0
      Coeff_end$Coeff_final<-matrix(Coeff_m,ncol=1)
      } else {
       Coeff_end$Coeff_final<-Coeff_end$Coeff_raw }
  }
 
  colnames(Coeff_end)[1]<-"Age"
  
  return(Coeff_end) 
   
}

## Applying 'CoeffLed' function ##
dtaC<-ddply(dtaP,.(Year,Sex), function(i) CoeffLed (X=i[,]))

#head(dtaC);tail(dtaC);dim(dtaC)
#write.csv(dtaC,"Coeff French dept.csv",row.names=F)


## Step 3. Applying obtained coefficients, getting new death counts ##

## Adding to the main dataset the obtained Coeff ##
dtaC1<-dtaC[,c("Year","Sex","Age","Cause","Coeff_final")]
dd<-merge(dta,dtaC1,all.x=T)

## Function redistributing ill-defined causes using the Ledermann's method ##
## X - data frame on which the operation should be performed
## FROM - what to redistribute (should be the same as in 'CoeffLed' function above) ##
## TO - where to redistribute (should be the same as in 'CoeffLed' function above) ##
## Iterations - number of iterations in the Ledermann's loop (4 by default) ##
CoeffLedRedist<-function (X,FROM=10,TO=1:9,Iterations=4){
  
  ## SILD ##
  tmp<-subset(X,Cause %in% FROM,select=c(Cause,Region,Deaths)) 
  tmp<-aggregate(tmp$Deaths,by=list(Region=tmp$Region),na.rm=TRUE,FUN=sum)
  #names(tmp)<-c("Region","SILD")
  
  tmp1<-subset(X,Cause %in% TO,select=c(Cause,Region,Deaths,Coeff_final)) 
  DNatSC<-aggregate(tmp1$Deaths,by=list(Cause=tmp1$Cause),na.rm=TRUE,FUN=sum) ## Total national by cause without SILD
  DNatNSTot<-subset(X,Cause %in% c(FROM),select=Deaths) ## Ill-defined national 
  DNatNSTot<-sum(DNatNSTot$Deaths,na.rm=TRUE) 
  
  Coeff<-subset(tmp1,Region==1,select=c(Cause,Coeff_final))## vector of coefficients same for all regions 
  Coeff<-Coeff[order(Coeff$Cause),]; #Coeff$Coeff[is.na(Coeff$Coeff)]<-0
  DNatNSC<-Coeff$Coeff_final*DNatNSTot ## Distribution of total ill-defined (national) counts by cause
  
  DNatTotC<-DNatSC[,2]+DNatNSC ## Total national by cause after adding ill-defined
  
  CoeffDiss<-DNatNSC/DNatTotC ## Proportion (share) of ill-defined causes in each cause incl. SILD
  CoeffDiss[CoeffDiss=='NaN']<-0;  CoeffDiss<-as.data.frame(cbind(TO,CoeffDiss))
  names(CoeffDiss)<-c("Cause","CoeffDiss")
  
  out<-NULL
  for (i in sort(unique(tmp1$Region))){
    DNS<-tmp[tmp$Region==i,2]## SILD total to be redistributed
    xx<-subset(tmp1,Region==i)
    xx<-merge(xx,CoeffDiss)
    xx<-within(xx,Final<-Deaths)
    Check<-sum(xx$Coeff_final*xx$Deaths)
    if(Check==0)
    {
      xx$Final<-xx$Final+xx$Coeff_final*DNS
      xx$Pond<-0
      xx$PondComp<-0
      xx$SILD<-0
      }else{
      for(k in 1:Iterations) { ## Ledermann's loop
        xx<-within(xx,Pond<-Final*CoeffDiss)
        xx<-within(xx,PondComp<-Pond/(sum(Pond)+0.000001))
        xx<-within(xx,SILD<-DNS*PondComp)
        xx<-within(xx,Final<-SILD+Deaths)     
      }
    }
    out<-rbind(out,xx)    }
  out[,4:dim(out)[2]]<-round(out[,4:dim(out)[2]],3)
  
  return(out)   }

## Applying 'CoeffLedRedist' function ##
dtaLM<-ddply(dd,.(Year,Sex,Age), function(i) CoeffLedRedist (X=i[,]))
#head(dtaLM)
#tail(dtaLM) ## 'Final' - death counts after the LM redistribution

#write.csv(dtaLM, "Output LM.csv", row.names=F)


### II. Alternative method: simple proportional redistribution (PR) ##
dd<-subset(dta1, Region!=999,select=-c(RegionName,CauseName,AgeLabel)) ## national total will be recalculated afterwards

## Function redistributing given cause(s) proportionally across other causes  ##
## X - data frame   
## FROM - what to redistribute
## TO - where to redistribute ##
Redist<-function (X,FROM,TO){
  d0<-subset(X,!Cause %in% c(TO,FROM),select=-c(Year,Region,Sex,Age)) # subset which should remain unchanged
  d1<-subset(X,Cause %in% TO,select=-c(Year,Region,Sex,Age)) 
  d2<-subset(X,Cause %in% FROM,select=-c(Year,Region,Sex,Age))
  nn<-2 ##  index of the column with death counts
  add<-sum(d2[,nn]) ## sum of CoD to be redistributed
  share<-d1
  share[,nn]<-share[,nn]/(sum(share[,nn])+0.000001) ## in case of all zeros
  out<-d1;  out[,nn]<-out[,nn]+share[,nn]*add
  d2[,nn]<-0
  out<-rbind(d0,out,d2)
  out[,nn]<-round(out[,nn],3)
  return(out) }

## Applying 'Redistr' function ##
dta2<-ddply(dd,.(Year,Region,Sex,Age), function(i) Redist (X=i[,],FROM=10,TO=1:9))

## National total##
tmp<-aggregate(dta2$Deaths, by=list(Year=dta2$Year,Sex=dta2$Sex,
                                    Age=dta2$Age,Cause=dta2$Cause), FUN=sum)
names(tmp)[5]<-"Deaths"
tmp$Region<-999
dta2<-rbind(dta2,tmp)
names(dta2)[6]<-"DeathsProportional"
head(dta2)

## III. Preparing the output dataset ##
tmp<-subset(dtaLM,select=c(Year,Sex,Age,Cause,Region,Final))

## Adding national total (all regions combined) ## 
tmp1<-aggregate(tmp[,6],by=list(Year=tmp$Year,Sex=tmp$Sex,Cause=tmp$Cause,Age=tmp$Age),FUN=sum)
names(tmp1)[5]<-"Final"
tmp1$Region<-999

tmp<-rbind(tmp,tmp1)
names(tmp)[6]<-"DeathsLedermann"

output<-merge(dta1,tmp,all.x=T) ## keeping item 10 (ill-defined) in the original dataset
output<-merge(output,dta2)
output<-output[order(output$Region,output$Cause,output$Age),]

## IV. Checking the sum of deaths (all causes combined) before and after the redistribution ##

DF<-aggregate(output[,9:11],by=list(Year=output$Year,Region=output$Region,Sex=output$Sex,
                                    Age=output$Age),FUN=sum,na.rm=T)
head(DF)

DF<-within(DF, DifferenceLM<-Deaths-DeathsLedermann)
DF<-within(DF, DifferencePR<-Deaths-DeathsProportional)
range(DF$DifferenceLM) 
range(DF$DifferencePR)

## Saving the output ##
write.csv (output, "Output.csv", row.names=F)


## V. Calculating mx and age-standardized death rate (SDR) ##

## Population exposures ## 
pop<-as.data.frame(read_xlsx("Population.xlsx"))

### Merging death counts and population ##
out<-merge(output, pop, all.x=T)
out$Deaths[is.na(out$Deaths)]<-0
out$DeathsLedermann[is.na(out$DeathsLedermann)]<-0
out$DeathsProportional[is.na(out$Proportional)]<-0

## Age-cause-specific mortality rates mx ##
out<-within(out, mx<-Deaths/Pop)
out<-within(out,mxLM<-DeathsLedermann/Pop)
out<-within(out,mxPR<-DeathsProportional/Pop)

## Population Standard (1976) ##
Wx<-c(0.016, 0.064,rep(0.14,5),0.11,0.07,0.03,0.0092,0.0008)
sum(Wx) ## check
length(Wx) ## number of groups
length(unique(out$Age))

## Function to get SDR ##
getSDR<-function (mx){
  SDR<-sum(mx*Wx)*100000
  names(SDR)<-"SDR"
  return (SDR) }

## Before redistribution ##
SDR<-ddply(out,.(Year,Region,Sex, Cause, CauseName),function (i) getSDR (i$mx))
## Ledermann ##
SDR2<-ddply(out,.(Year,Region,Sex, Cause, CauseName),function (i) getSDR (i$mxLM))
names(SDR2)[6]<-"SDRLM"
## Proportional ##
SDR3<-ddply(out,.(Year,Region,Sex, Cause, CauseName),function (i) getSDR (i$mxPR))
names(SDR3)[6]<-"SDRPR"

dim(SDR); dim(SDR2); dim(SDR3)

SDR<-merge(SDR, SDR2); SDR<-merge(SDR, SDR3)

SDR<-SDR[order(SDR$Year,SDR$Region,SDR$Sex,SDR$Cause),]
#head(SDR)

write.csv (SDR, "SDR.csv", row.names=F)

######################  END ########################################
