                            ## Simulation test.R ##

## Demography: research note ##
## Method for redistributing misclassified causes of death ##
## Online Supplementary Appendix S4 ##
## Validation of the Ledermanns model: a simulation test ##

## Testing the Ledermann's method using simulated data ##
## Data2: sim. dataset with ill-defined (Y4) causes to be redistributed across well-defined Y1-Y3  
## Data1: sim. dataset without ill-defined causes with variables Z1-Z3 
rm(list=objects())
library(MASS)
library(plyr)
library(Metrics)

## Set your working directory here ##
setwd("~/")

## I. Simulating data ##

# For this simulation, we consider 3 causes of death.

### Step 1: Choose the number of observation. Here, 95 French departments.
N<-95

### Step 2: CREATE Data1: the "true" but unobservable share of death by causes
  # Z1 follows a normal distribution of mean 10 and variance 1.
  # Z2 follows a normal distribtuion of mean 40 and variance 1.
  # covariance matrix = Idt matrix
Data1 <- as.data.frame("colnames<-"(mvrnorm(n=N, mu = c(10,40), Sigma = diag(2)),c("Z1", "Z2")))
  #Z3 is set in order to have Z1+Z2+Z3=1
Data1$Z3  <- 100-Data1$Z1  - Data1$Z2 

### Step 3: Simulate the coefficients teta.
  #teta_2 and teta_3 follows normal distribution with small variance
teta_2=pmax(rnorm(n=N, mean=0.025, sd=0.04),0)
teta_3=pmax(rnorm(n=N, mean=0.1, sd=0.05),0)
  #teta_3 is set in order to have cov(X,Zi)=0 with X=ill-defined
teta_1=pmax((teta_3-teta_2)*(cov(Data1$Z1,Data1$Z2)/var(Data1$Z1))+teta_3)

### Step 4: deduce the other variables: ill-defined and Yi the observed share of death
#We call Y4 the share of ill-defined
Y4<-(teta_1*Data1$Z1+teta_2*Data1$Z2+teta_3*Data1$Z3)

Y1 <- (1-teta_1)*Data1$Z1
Y2 <- (1-teta_2)*Data1$Z2
Y3 <- (1-teta_3)*Data1$Z3

Data2 <- data.frame(Y1,Y2,Y3,Y4) ## the observed dataset

### Step 5: plot the Ledermann regression
#set the cause of death of interest
#var<-Y2
#plot(Data2$Y4, Data2$Y2, main = "Main title", xlab = "var", ylab = "Ill-defined")
#abline(lm(var~Y4, data = mtcars), col = "blue")
#model<-lm(Y4~var)
#summary(model)

## II. Testing ##
dta1<-Data1
dta2<-Data2

## Calculating the share of each cause in the total (all causes combined)  ## 
dta2P<-as.data.frame(t(apply(dta2,1,prop.table)))


## Getting dissimulation coefficients ##
## X - a given CoD 
## Y - ill-defined 
CoeffLed<-function (X,Y){
      model<-lm(X~Y) 
      Reg<-summary(model)[[4]]
      slope<-round(Reg[2,1],5)
      if (slope<0){
        Coeff<-slope
        } else{Coeff<-0}
      return(Coeff) }

C1<-CoeffLed (dta2P$Y1,dta2P$Y4) ## coeff for Y1
C2<-CoeffLed (dta2P$Y2,dta2P$Y4) ## coeff for Y2
C3<-CoeffLed (dta2P$Y3,dta2P$Y4) ## coeff for Y3
C1+C2+C3

## After Ledermann's redistribution ##
dta2<-within(dta2,Y1_LM<-Y1+Y4*(-C1))
dta2<-within(dta2,Y2_LM<-Y2+Y4*(-C2))
dta2<-within(dta2,Y3_LM<-Y3+Y4*(-C3))

## Check LM ##
dta2<-within(dta2, CheckLM<-round((Y1+Y2+Y3+Y4)-(Y1_LM+Y2_LM+Y3_LM)),2)
head(dta2)

## After proportional redistribution ##
dta2<-within(dta2,TOT<-Y1+Y2+Y3) ## Total without ill-defined 
dta2<-within(dta2,Y1_PR<-Y1+Y1/TOT*Y4)
dta2<-within(dta2,Y2_PR<-Y2+Y2/TOT*Y4)
dta2<-within(dta2,Y3_PR<-Y3+Y3/TOT*Y4)

## Check PR ##
dta2<-within(dta2, CheckPR<-round((Y1+Y2+Y3+Y4)-(Y1_PR+Y2_PR+Y3_PR)),2)
head(dta2)

## Comparing against the simulated data without ill-defined causes ##
dta2$Region<-1:N; dta1$Region<-1:N

out<-merge(dta2,dta1)
head(out)

## Model fit ##

## Cause Y1 ## 

## LM ##
plot(out$Y1_LM, out$Z1, main = "Ledermann",col="darkgreen",
     xlab = "estimated", ylab = "observed",
     pch = 19, frame = FALSE)
model_led1<-lm(out$Y1_LM~out$Z1)
summary(model_led1)
rmse(out$Z1,out$Y1_LM)


## PR ##
plot(out$Y1_PR, out$Z1, main = "Proportional",col="darkblue",
     xlab = "estimated", ylab = "observed",
     pch = 19, frame = FALSE)
model_pr1<-lm(out$Y1_PR~out$Z1)
summary(model_pr1)
rmse(out$Z1,out$Y1_PR)


## Y2 ## 

## LM ##
plot(out$Y2_LM, out$Z2, main = "Ledermann",col="darkgreen",
     xlab = "estimated", ylab = "observed",
     pch = 19, frame = FALSE)
model_led2<-lm(out$Y2_LM~out$Z2)
summary(model_led2)
rmse(out$Z2,out$Y2_LM)

## PR ##
plot(out$Y2_PR, out$Z2, main = "Proportional",col="darkblue",
     xlab = "estimated", ylab = "observed",
     pch = 19, frame = FALSE)
model_pr2<-lm(out$Y2_PR~out$Z2)
summary(model_pr2)
rmse(out$Z2,out$Y2_PR)


## Y3 ## 

## LM ##
plot(out$Y3_LM, out$Z3, main = "Ledermann",col="darkgreen",
     xlab = "estimated", ylab = "observed",
     pch = 19, frame = FALSE)
model_led3<-lm(out$Z3~out$Y3_LM)
summary(model_led3)
rmse(out$Z3,out$Y3_LM)


## PR ##
plot(out$Y3_PR, out$Z3, main = "Proportional",col="darkblue",
     xlab = "estimated", ylab = "observed",
     pch = 19, frame = FALSE)
model_pr3<-lm(out$Z3~out$Y3_PR)
summary(model_pr3)
nobs(model)
rmse(out$Z3,out$Y3_PR)


## Function plotting observed vs. predicted values ##
## with additional statistics ##

x<-44:54
y<-x

pdf("Simulation_low_theta.pdf",width=16, height=10, pointsize=12)
x11(width=16, height=8, pointsize=12) 
m <- matrix(c(1,2,3),nrow=1,byrow=TRUE)
layout(m,widths=c(1,1,1),heights=c(1))
  par(oma=c(1,0,0,0),mar=c(4,5,3.2,0.5),bg="white")

##y_1
plot(out$Z1, out$Y1_LM, main = "Cause of death 1",col="darkgreen",
     xlab = "%CoD 1 in the real data", ylab = "%CoD 1 in the estimated data",
     ylim=c(6,14),xlim=c(6,14),
     pch = 19, frame = FALSE, xaxt="n",yaxt="n", cex.main=2.5, cex.lab=2.5 )
axis(1,at=seq(6,14,2),lab=seq(6,14,2),cex.axis=2.5,font.axis=2)
axis(2,at=seq(6,14,2),lab=seq(6,14,2),cex.axis=2.5,font.axis=2)
abline(lm(out$Y1_LM~out$Z1),col="darkgreen")
points(out$Z1, out$Y1_PR, col="darkblue")
abline(lm(out$Y1_PR~out$Z1), col="darkblue")
abline(lm(x~y), col="black", lty=2)
abline(h=seq(6,14,2),lty=3,col="grey")
abline(v=seq(6,14,2),lty=3,col="grey")

legend(x="topleft",lty=c(1,1,2),lwd=2,col=c("darkgreen","darkblue","black"),cex=1.6,text.font=2,
       legend=c("Ledermann","Proportional","Perfect fit"),ncol=1,bty="n")
text(11.5, 9, expression(RMSE), col = "darkblue", cex=2)
text(13.25, 9, round(rmse(out$Z1,out$Y1_PR),digits=2), col = "darkblue", cex=2)
text(11.5, 8.5, expression(R2) , col = "darkblue", cex=2)
text(13.25, 8.5, round(summary(model_pr1)$adj.r.square,2), col = "darkblue", cex=2)
text(11.5, 8, expression(Fstat) , col = "darkblue", cex=2)
text(13.25, 8, round(summary(model_pr1)$fstatistic[1]), col = "darkblue", cex=2)

text(6.5, 12, expression(RMSE), col = "darkgreen", cex=2)
text(8.25, 12, round(rmse(out$Z1,out$Y1_LM),digits=2), col = "darkgreen", cex=2)
text(6.5, 11.5, expression(R2) , col = "darkgreen", cex=2)
text(8.25, 11.5, round(summary(model_led1)$adj.r.square,2), col = "darkgreen", cex=2)
text(6.5, 11, expression(Fstat) , col = "darkgreen", cex=2)
text(8.25, 11, round(summary(model_led1)$fstatistic[1]), col = "darkgreen", cex=2)

##y_2
plot(out$Z2, out$Y2_LM, main = "Cause of death 2",col="darkgreen",
     xlab = "%CoD 2 in the real data", ylab = "%CoD 1 in the estimated data",
     ylim=c(35,43),xlim=c(35,43),
     pch = 19, frame = FALSE, xaxt="n",yaxt="n", cex.main=2.5, cex.lab=2.5 )
axis(1,at=seq(35,43,2),lab=seq(35,43,2),cex.axis=2.5,font.axis=2)
axis(2,at=seq(35,43,2),lab=seq(35,43,2),cex.axis=2.5,font.axis=2)
abline(lm(out$Y2_LM~out$Z2),col="darkgreen")
points(out$Z2, out$Y2_PR, col="darkblue")
abline(lm(out$Y2_PR~out$Z2), col="darkblue")
abline(lm(x~y), col="black", lty=2)
abline(h=seq(35,43,2),lty=3,col="grey")
abline(v=seq(35,43,2),lty=3,col="grey")

text(36, 43, expression(RMSE), col = "darkblue", cex=2)
text(37.75, 43, round(rmse(out$Z2,out$Y2_PR),digits=2), col = "darkblue", cex=2)
text(36, 42.5, expression(R2) , col = "darkblue", cex=2)
text(37.75, 42.5, round(summary(model_pr2)$adj.r.square,2), col = "darkblue", cex=2)
text(36, 42, expression(Fstat) , col = "darkblue", cex=2)
text(37.75, 42, round(summary(model_pr2)$fstatistic[1]), col = "darkblue", cex=2)

text(40,37, expression(RMSE), col = "darkgreen", cex=2)
text(41.75,37, round(rmse(out$Z2,out$Y2_LM),digits=2), col = "darkgreen", cex=2)
text(40,36.5, expression(R2) , col = "darkgreen", cex=2)
text(41.75,36.5, round(summary(model_led2)$adj.r.square,2), col = "darkgreen", cex=2)
text(40,36, expression(Fstat) , col = "darkgreen", cex=2)
text(41.75,36, round(summary(model_led2)$fstatistic[1]), col = "darkgreen", cex=2)
##y_3
plot(out$Z3, out$Y3_LM, main = "Cause of death 3",col="darkgreen",
     xlab = "%CoD 3 in the real data", ylab = "%CoD 1 in the estimated data",
     ylim=c(46,54),xlim=c(46,54),
     pch = 19, frame = FALSE, xaxt="n",yaxt="n" , cex.main=2.5, cex.lab=2.5 )
axis(1,at=seq(46,54,2),lab=seq(46,54,2),cex.axis=2.5,font.axis=2)
axis(2,at=seq(46,54,2),lab=seq(46,54,2),cex.axis=2.5,font.axis=2)
abline(lm(out$Y3_LM~out$Z3),col="darkgreen")
points( out$Z3, out$Y3_PR, col="darkblue")
abline(lm(out$Y3_PR~out$Z3), col="darkblue")
abline(lm(x~y), col="black", lty=2)  
abline(h=seq(46,54,2),lty=3,col="grey")
abline(v=seq(46,54,2),lty=3,col="grey")

text(52,48, expression(RMSE), col = "darkblue", cex=2)
text(53.75,48, round(rmse(out$Z3,out$Y3_PR),digits=2), col = "darkblue", cex=2)
text(52,47.5, expression(R2) , col = "darkblue", cex=2)
text(53.75,47.5, round(summary(model_pr3)$adj.r.square,2), col = "darkblue", cex=2)
text(52,47, expression(Fstat) , col = "darkblue", cex=2)
text(53.75,47, round(summary(model_pr3)$fstatistic[1]), col = "darkblue", cex=2)

text(47, 53, expression(RMSE), col = "darkgreen", cex=2)
text(48.75, 53, round(rmse(out$Z3,out$Y3_LM),digits=2), col = "darkgreen", cex=2)
text(47, 52.5, expression(R2) , col = "darkgreen", cex=2)
text(48.75, 52.5, round(summary(model_led3)$adj.r.square,2), col = "darkgreen", cex=2)
text(47, 52, expression(Fstat) , col = "darkgreen", cex=2)
text(48.75, 52, round(summary(model_led3)$fstatistic[1]), col = "darkgreen", cex=2)

dev.off()
############################################# END ###################################################
